% Computes magnetic field variation across a set of dipoles
% February 2009 Geophysics 223

% SI units used.  Both Earth and induced fields in nT

clear all
close all

% Set initial inclination (degrees)
inc_deg =90;
rad = 180/pi;
inc = (90-inc_deg)/rad
% inc_deg = 90 is a vertical B field

%  This version models more than 1 dipole


ndipole = 2;
d=[5,5];     % Depth of centre of spheres (m)
x_dipole = [-3,3];   % Horizontal position of spheres (m)

% Example in 5.1
%ndipole = 2;
%d=[4,4];
%x_dipole = [-3,3];




B=50000; % Strength of Earth's field (nT)
scale = 1000; % Controls vertical scale on plot of magnetic anomaly
k=0.015; % Susceptibility of sphere (SI)
a =2;    % Radius of spheres (m)
vol = 4*pi*a*a*a/3;
mu_0 = 4*pi*1e-7;
M=vol*k*B/mu_0   % Magnetic moment of sphere (ignores demagnetizing effect)

z_obs = 0.1; % Elevation at which measure (negative is up)

xlim = 15;  % Horizontal plot limits (m)
%Define vectors for geometry plot
ground_x = [-xlim,xlim];   ground_z = [0,0];    b_bg =[B,B];



% Start loop to vary inclination and depth
irun =1
while irun ~= -1
 
  b_earth_z =  -B*cos(inc); b_earth_x =  -B*sin(inc);
  datum = [z_obs,z_obs]
  % Define vector for plotting inclination 
  
  % M in sphere
  for idipole = 1:ndipole
    bplotx1(idipole,1) = [x_dipole(idipole) + 0.5*a*sin(inc)];
    bplotx1(idipole,2) = [x_dipole(idipole)-0.5*a*sin(inc)];
    bplotz1(idipole,1) = [-d(idipole)+0.5*a*cos(inc)];
    bplotz1(idipole,2) = [-d(idipole)-0.5*a*cos(inc)];
  end
  
  % Earth's field at surface
  bplotx2 = [10+1.5*sin(inc),10-1.5*sin(inc)];
  bplotz2 = [2+1.5*cos(inc),2-1.5*cos(inc)];

  npts =100;
  for idipole =1:ndipole
    for ipts = 1:npts+1
      sphere_x(ipts,idipole) = x_dipole(idipole)-a*cos(2*pi*(ipts-1)/npts);
      sphere_z(ipts,idipole) = -d(idipole)-a*sin(2*pi*(ipts-1)/npts);   
    end
  end
  
  % Define points where H is computed on z=0 surface
  ix=0;
  for xdum = -xlim:0.2:xlim
    ix=ix+1;
    x(ix) =xdum;
  end
  nx=ix;

  
 for ix =1:nx
    b_x_total(ix)= b_earth_x;
    b_z_total(ix)= b_earth_z;
  
    for idipole = 1:ndipole  
      r=sqrt((d(idipole)-z_obs)*(d(idipole)-z_obs)+(x(ix)-x_dipole(idipole))*(x(ix)-x_dipole(idipole)));
      theta = atan((x(ix)-x_dipole(idipole))/(d(idipole)-z_obs));
      
      % Dipole fields
      b_r(ix)=-(mu_0*2*M*cos(inc-theta))/(r^3);
      b_theta(ix)=-(mu_0*M*sin(inc-theta))/(r^3);
    
      b_z(ix)= b_r(ix)*cos(theta)+b_theta(ix)*sin(theta);
      b_x(ix)= b_r(ix)*sin(theta)-b_theta(ix)*cos(theta);
 
      % Sum of dipole and Earth's fields  
      b_x_total(ix)=b_x_total(ix)+ b_x(ix);
      b_z_total(ix)=b_z_total(ix)+ b_z(ix);
    end
    b_total(ix) = sqrt(b_x_total(ix)*b_x_total(ix)+b_z_total(ix)*b_z_total(ix));
 end


  bmax = max(b_total);
  bmin = min(b_total);
  bd = 0.1*(max(b_total)- min(b_total));
 
  % Plot magnetic field response
  subplot (2,1,1)
  plot(x,b_total)
  hold on
  plot(ground_x,b_bg,':')
  hold off
  %plot(x,h_z_total,'-')
  xlabel ('x(m)')
  ylabel ('B_T (nT)')
  title ('Total magnetic field')
  axis([-xlim,xlim,bmin-bd,bmax+bd])
  
  %Plot model geometry and inclination
  subplot (2,1,2)
  plot(ground_x,ground_z)
  axis([-xlim,xlim, -10,10])
  hold on
    plot(ground_x,-datum,':')
    for idipole=1:ndipole
      plot(sphere_x(:,idipole),sphere_z(:,idipole));
      plot(bplotx1(idipole,:),bplotz1(idipole,:))
      plot(bplotx1(idipole,2),bplotz1(idipole,2),'.')
      
    end
    plot(bplotx2,bplotz2)  
    plot(bplotx2(2),bplotz2(2),'.')
    xlabel ('x(m)')
    ylabel ('z(m)')
  hold off
  
  
% Change these parameters if desired 
 choice=menu('Options', 'Increase i','Decrease i','Raise measurement level','Lower measurement level','Quit');
  
 if choice ==1 ;inc_deg = inc_deg+5; inc = (90-inc_deg)/rad;end
 if choice ==2 ; inc_deg = inc_deg-5; inc = (90-inc_deg)/rad;  end
 if choice ==3;  z_obs = z_obs -1;   end 
 if choice ==4;  z_obs = z_obs +1;   end   
 
 if choice > 4; irun=-1; print -dpsc  mag_dipole_multi_v1.ps; end
 
 
 % Write out results
 
 for i = 1:nx
  d_out(i,1) = x(i);
  d_out(i,2) = b_total(i);
end
    
save b_total.txt d_out -ASCII
 
 
 
 
end